@@ -1,5 +1,5 @@
 /*
-  Copyright 2016 David Robillard <http://drobilla.net>
+  Copyright 2016 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -46,6 +46,21 @@
 #    define ZIX_API
 #    define ZIX_PRIVATE static
 #endif
+
+#ifdef __GNUC__
+#    define ZIX_PURE_FUNC __attribute__((pure))
+#    define ZIX_CONST_FUNC __attribute__((const))
+#    define ZIX_MALLOC_FUNC __attribute__((malloc))
+#else
+#    define ZIX_PURE_FUNC
+#    define ZIX_CONST_FUNC
+#    define ZIX_MALLOC_FUNC
+#endif
+
+#define ZIX_PURE_API ZIX_API ZIX_PURE_FUNC
+#define ZIX_CONST_API ZIX_API ZIX_CONST_FUNC
+#define ZIX_MALLOC_API ZIX_API ZIX_MALLOC_FUNC
+
 /** @endcond */
 
 #ifdef __cplusplus
